@@ -1,16 +1,9 @@
 package MooseX::Traits;
 use Moose::Role;
 
-our $VERSION   = '0.01';
+our $VERSION   = '0.02';
 our $AUTHORITY = 'id:JROCKWAY';
 
-has 'traits' => (
-    is         => 'ro',
-    isa        => 'ArrayRef[Str]',
-    auto_deref => 1,
-    default    => sub { [] },
-);
-
 has '_trait_namespace' => (
     # no accessors or init_arg
     isa => 'Str',
@@ -18,35 +11,39 @@ has '_trait_namespace' => (
 
 # dont pollute the consuming class with methods they don't want
 my $transform_trait = sub {
-    my ($self, $name) = @_;
-    my $base = $self->meta->get_attribute('_trait_namespace')->get_value($self);
-    return $name unless $base;
+    my ($class, $name) = @_;
+    my $namespace = $class->meta->get_attribute('_trait_namespace');
+    my $base;
+    if($namespace->has_default){
+        $base = $namespace->default;
+        if(ref $base eq 'CODE'){
+            $base = $base->();
+        }
+    }
 
+    return $name unless $base;
     return $1 if $name =~ /^[+](.+)$/;
     return join '::', $base, $name;
 };
 
-around new => sub {
-    my ($next, $class, @args) = @_;
-    
-    my $self= $class->$next(@args);
+sub new_with_traits {
+    my ($class, %args) = @_;
     
     my @traits = 
       grep { Class::MOP::load_class($_); 1 }
-        map { $self->$transform_trait($_) } $self->traits;
+        map { $class->$transform_trait($_) } @{ delete $args{traits} || [] };
 
     my @anon_args = (
-        superclasses => [$self->meta->name],
+        superclasses => [$class->meta->name],
         cache        => 1,
     );
     push @anon_args, roles => [@traits] if @traits;
 
-    my $metaclass = $self->meta->meta->name;    
+    my $metaclass = $class->meta->meta->name;    
     my $new_class = $metaclass->create_anon_class(@anon_args);
-    $new_class->rebless_instance($self, @args);
 
-    return $self;
-};
+    return $new_class->name->new(%args);
+}
 
 1;
 
@@ -72,7 +69,7 @@ And a class:
 
 Apply the roles to the class at C<new> time:
 
-  my $class = Class->new( traits => ['Role'], foo => 42 );
+  my $class = Class->new_with_traits( traits => ['Role'], foo => 42 );
 
 Then use your customized class:
 
@@ -87,10 +84,10 @@ arbitrarily.  This module makes it easy for the end user to use these
 components.  Instead of requiring the user to create a named class
 with the desired roles applied, or applying roles to the instance
 one-by-one, he can just pass a C<traits> parameter to the class's
-constructor.  This role will then apply the roles in one go, and cache
-the resulting class (for efficiency).  Arguments meant to initialize
-the applied roles' attributes can also be passed to the constructor.
-(This will probably break L<MooseX::StrictConstructor>.)
+C<new_with_traits> constructor.  This role will then apply the roles
+in one go, cache the resulting class (for efficiency), and return a
+new instance.  Arguments meant to initialize the applied roles'
+attributes can also be passed to the constructor.
 
 =head1 ATTRIBUTES YOUR CLASS GETS
 
@@ -122,25 +119,19 @@ Example:
   with 'MooseX::Traits';
   has '+_trait_namespace' => ( default => 'Another' );
  
-  my $instance = Another::Class->new(
+  my $instance = Another::Class->new_with_traits(
       traits => ['Trait'], # "Another::Trait", not "Trait"
       bar    => 'bar',
   );
   $instance->does('Trait')          # false
   $instance->does('Another::Trait') # true
 
-  my $instance2 = Another::Class->new(
+  my $instance2 = Another::Class->new_with_traits(
       traits => ['+Trait'], # "Trait", not "Another::Trait"
   );
   $instance2->does('Trait')          # true
   $instance2->does('Another::Trait') # false
 
-=head1 BUGS
-
-This role wraps C<new> with C<around>, which probably isn't the best
-idea ever conceived.  I might change this implementation detail at a
-later date.
-
 =head1 AUTHOR
 
 Jonathan Rockway C<< <jrockway@cpan.org> >>
